from cms.views import details as cms_view_details
from django.core.exceptions import PermissionDenied
from django.db.models import F
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse_lazy as reverse
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from ..forms.subjects import (
    CourseRegistrationForm, EventRegistrationForm, OrderableRegistrationForm,
    SubjectFilterForm, SubjectForm,
)
from ..models.courses import Course
from ..models.events import Event
from ..models.leprikonsite import LeprikonSite
from ..models.orderables import Orderable
from ..models.subjects import (
    Subject, SubjectPayment, SubjectRegistration, SubjectType,
)
from .generic import (
    ConfirmUpdateView, CreateView, DetailView, FilteredListView, ListView,
    UpdateView,
)


class SubjectTypeMixin:
    _models = {
        SubjectType.COURSE: Course,
        SubjectType.EVENT: Event,
        SubjectType.ORDERABLE: Orderable,
    }

    def dispatch(self, request, subject_type, *args, **kwargs):
        self.subject_type = get_object_or_404(SubjectType, slug=subject_type)
        self.model = self._models[self.subject_type.subject_type]
        return super().dispatch(request, *args, **kwargs)

    def get_placeholder(self):
        return super().get_placeholder() + ':' + self.subject_type.slug

    def get_queryset(self):
        return super().get_queryset().filter(subject_type=self.subject_type)

    def get_template_names(self):
        return [
            'leprikon/{}{}.html'.format(self.subject_type.slug, self.template_name_suffix),
            'leprikon/{}{}.html'.format(self.subject_type.subject_type, self.template_name_suffix),
            'leprikon/subject{}.html'.format(self.template_name_suffix),
        ]


class CMSSubjectTypeMixin(SubjectTypeMixin):
    def dispatch(self, request, *args, **kwargs):
        # Get current CMS Page
        cms_page = request.current_page.get_public_object()

        # Check whether it really is a Leprikon subject type application.
        # It might also be one of its sub-pages.
        if cms_page.application_urls != 'LeprikonSubjectTypeApp':
            # In such case show regular CMS Page
            return cms_view_details(request, *args, **kwargs)
        return super(CMSSubjectTypeMixin, self).dispatch(request, cms_page.application_namespace, **kwargs)


class SubjectListBaseView(FilteredListView):
    form_class = SubjectFilterForm
    preview_template = 'leprikon/subject_preview.html'
    paginate_by = 10

    def get_title(self):
        return _('{subject_type} in school year {school_year}').format(
            subject_type=self.subject_type.plural,
            school_year=self.request.school_year,
        )

    def get_message_empty(self):
        return _('No {subject_type} matching given search parameters found.').format(
            subject_type=self.subject_type.plural,
        )

    def get_form(self):
        return self.form_class(
            subject_type_type=self.subject_type.subject_type,
            subject_types=[self.subject_type],
            school_year=self.request.school_year,
            is_staff=self.request.user.is_staff,
            data=self.request.GET,
        )

    def get_queryset(self):
        return self.get_form().get_queryset()


class SubjectListView(CMSSubjectTypeMixin, SubjectListBaseView):
    pass


class SubjectListMineView(SubjectTypeMixin, SubjectListBaseView):

    def get_template_names(self):
        return [
            'leprikon/{}_list_mine.html'.format(self.subject_type.slug),
            'leprikon/{}_list_mine.html'.format(self.subject_type.subject_type),
            'leprikon/subject_list_mine.html',
        ] + super(SubjectListMineView, self).get_template_names()

    def get_title(self):
        return _('My {subject_type} in school year {school_year}').format(
            subject_type=self.subject_type.plural,
            school_year=self.request.school_year,
        )

    def get_queryset(self):
        return super(SubjectListMineView, self).get_queryset().filter(leaders=self.request.leader)


class SubjectDetailView(CMSSubjectTypeMixin, DetailView):

    def get_queryset(self):
        qs = super(SubjectDetailView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(public=True)
        return qs


class SubjectRegistrationsView(SubjectTypeMixin, DetailView):
    template_name_suffix = '_registrations'

    def get_queryset(self):
        qs = super(SubjectRegistrationsView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(leaders=self.request.leader)
        return qs

    def get_context_data(self, **kwargs):
        context = super(SubjectRegistrationsView, self).get_context_data(**kwargs)
        context['site'] = LeprikonSite.objects.get_current()
        return context


class SubjectUpdateView(SubjectTypeMixin, UpdateView):
    form_class = SubjectForm

    def get_title(self):
        return _('Change {subject_type} {subject}').format(
            subject_type=self.subject_type.name_akuzativ,
            subject=self.object.name,
        )

    def get_message(self):
        return _('The changes in {subject_type} {subject} have been saved.').format(
            subject_type=self.subject_type.name_genitiv,
            subject=self.object.name,
        )

    def get_queryset(self):
        qs = super(SubjectUpdateView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(leaders=self.request.leader)
        return qs


class SubjectMixin:
    def dispatch(self, request, pk, **kwargs):
        lookup_kwargs = {'subject_type': self.subject_type, 'id': int(pk)}
        if not self.request.user.is_staff:
            lookup_kwargs['public'] = True
        self.subject = get_object_or_404(Subject, **lookup_kwargs)
        if not self.subject.registration_allowed:
            return redirect(self.subject)
        self.request.school_year = self.subject.school_year
        return super().dispatch(request, **kwargs)


class SubjectRegistrationFormBaseView(CreateView):
    back_url = reverse('leprikon:registration_list')
    submit_label = _('Submit registration')
    message = _('The registration has been saved. We will inform you about its further processing.')
    template_name_suffix = '_registration_form'
    _form_classes = {
        SubjectType.COURSE: CourseRegistrationForm,
        SubjectType.EVENT: EventRegistrationForm,
        SubjectType.ORDERABLE: OrderableRegistrationForm,
    }

    def get_title(self):
        return _('Registration for {subject_type} {subject}').format(
            subject_type=self.subject_type.name_akuzativ,
            subject=self.subject.name,
        )

    def get_form_class(self):
        return self._form_classes[self.subject_type.subject_type]

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['subject'] = self.subject
        kwargs['user'] = self.request.user
        return kwargs


class SubjectRegistrationFormView(CMSSubjectTypeMixin, SubjectMixin, SubjectRegistrationFormBaseView):
    pass


class UserRegistrationMixin:
    _attrs = {
        SubjectType.COURSE: 'courseregistration',
        SubjectType.EVENT: 'eventregistration',
        SubjectType.ORDERABLE: 'orderableregistration',
    }

    def get_object(self):
        subject_registration = super().get_object()
        if subject_registration.user != self.request.user:
            raise PermissionDenied()
        return getattr(
            subject_registration,
            self._attrs[subject_registration.subject_type_type],
        )

    def get_queryset(self):
        return SubjectRegistration.objects.annotate(
            subject_type_type=F('subject__subject_type__subject_type'),
        )

    def get_template_names(self):
        return [self.template_name] if self.template_name else [
            'leprikon/{}_registration{}.html'.format(self.object.subject.subject_type.slug,
                                                     self.template_name_suffix),
            'leprikon/{}_registration{}.html'.format(self.object.subject.subject_type.subject_type,
                                                     self.template_name_suffix),
            'leprikon/subject_registration{}.html'.format(self.template_name_suffix),
        ]


class PDFMixin:
    event = 'pdf'

    def get(self, request, *args, **kwargs):
        obj = self.get_object()
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(obj.get_pdf_filename(self.event))
        return obj.write_pdf(self.event, response)


class SubjectRegistrationPdfView(PDFMixin, UserRegistrationMixin, DetailView):
    pass


class SubjectRegistrationPaymentRequestView(PDFMixin, UserRegistrationMixin, DetailView):
    event = 'payment_request'


class SubjectRegistrationCancelView(UserRegistrationMixin, ConfirmUpdateView):
    template_name_suffix = '_cancel'
    title = _('Registration cancellation request')

    def get_queryset(self):
        return super().get_queryset().filter(canceled=None)

    def get_question(self):
        return _('Are you sure You want to cancel the registration "{}"?').format(self.object)

    def get_message(self):
        return _('The cancellation request for {} has been saved.').format(self.object)

    def confirmed(self):
        if self.object.approved:
            self.object.cancelation_requested = now()
            self.object.cancelation_requested_by = self.request.user
            self.object.save()
        else:
            self.object.refuse(self.request.user)


class UserPaymentMixin:
    model = SubjectPayment

    def get_queryset(self):
        return super(UserPaymentMixin, self).get_queryset().filter(
            registration__user=self.request.user
        ).order_by('-accounted')


class SubjectPaymentsListView(UserPaymentMixin, ListView):
    template_name = 'leprikon/payments.html'
    paginate_by = 20

    def get_title(self):
        return _('Payments')


class SubjectPaymentPdfView(PDFMixin, UserPaymentMixin, DetailView):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['leprikon',
 'leprikon.admin',
 'leprikon.api',
 'leprikon.cms_plugins',
 'leprikon.conf',
 'leprikon.forms',
 'leprikon.forms.reports',
 'leprikon.management',
 'leprikon.management.commands',
 'leprikon.migrations',
 'leprikon.models',
 'leprikon.search_indexes',
 'leprikon.site',
 'leprikon.templatetags',
 'leprikon.views',
 'leprikon.views.reports']

package_data = \
{'': ['*'],
 'leprikon': ['locale/cs/LC_MESSAGES/*',
              'locale/en/LC_MESSAGES/*',
              'static/*',
              'static/admin/css/*',
              'static/admin/js/admin/*',
              'static/leprikon/css/*',
              'static/leprikon/fonts/*',
              'static/leprikon/img/*',
              'static/leprikon/js/*',
              'templates/admin/*',
              'templates/admin/inc/*',
              'templates/admin/leprikon/*',
              'templates/admin/leprikon/coursediscount/*',
              'templates/admin/leprikon/eventdiscount/*',
              'templates/admin/leprikon/message/*',
              'templates/admin/leprikon/messagerecipient/*',
              'templates/admin/leprikon/orderablediscount/*',
              'templates/admin/leprikon/subjectpayment/*',
              'templates/auth/widgets/*',
              'templates/cms/toolbar/items/*',
              'templates/leprikon/*',
              'templates/leprikon/admin/*',
              'templates/leprikon/cms/*',
              'templates/leprikon/cms/course/*',
              'templates/leprikon/cms/course_list/*',
              'templates/leprikon/cms/event/*',
              'templates/leprikon/cms/event_list/*',
              'templates/leprikon/cms/leader/*',
              'templates/leprikon/cms/leader_list/*',
              'templates/leprikon/cms/orderable/*',
              'templates/leprikon/cms/orderable_list/*',
              'templates/leprikon/payment_pdf/*',
              'templates/leprikon/payment_received/*',
              'templates/leprikon/registration_approved/*',
              'templates/leprikon/registration_canceled/*',
              'templates/leprikon/registration_payment_request/*',
              'templates/leprikon/registration_pdf/*',
              'templates/leprikon/registration_received/*',
              'templates/leprikon/registration_refused/*',
              'templates/leprikon/reports/*',
              'templates/leprikon/static/*',
              'templates/leprikon/subject_journal_pdf/*',
              'templates/leprikon/widgets/*',
              'templates/rocketchat/*']}

install_requires = \
['Django>=3.1.4,<4.0.0',
 'django-bankreader>=0.6.1,<0.7.0',
 'django-cms>=3.8.0,<4.0.0',
 'django-cron>=0.5.1,<0.6.0',
 'django-filer>=2.0.2,<3.0.0',
 'django-haystack>=3.0,<4.0',
 'django-localflavor>=3.0.1,<4.0.0',
 'django-mathfilters>=1.0.0,<2.0.0',
 'django-pays>=0.1.0,<0.2.0',
 'django-qr-code>=2.0.1,<3.0.0',
 'django-redis>=4.12.1,<5.0.0',
 'djangocms-link>=3.0.0,<4.0.0',
 'djangocms-snippet>=3.0.0,<4.0.0',
 'djangocms-style>=3.0.0,<4.0.0',
 'djangocms-text-ckeditor>=4.0.0,<5.0.0',
 'djangocms-video>=3.0.0,<4.0.0',
 'html2rml>=0.3.0,<0.4.0',
 'ipython>=7.19.0,<8.0.0',
 'pyexcel-xlsxw>=0.6.1,<0.7.0',
 'pymongo>=3.11.2,<4.0.0',
 'rocketchat-API>=1.12.1,<2.0.0',
 'sentry-sdk>=0.19.5,<0.20.0',
 'social-auth-app-django>=4.0.0,<5.0.0',
 'trml2pdf>=0.5.0,<0.6.0']

setup_kwargs = {
    'name': 'leprikon',
    'version': '3.0.0',
    'description': 'Django CMS based IS for education',
    'long_description': None,
    'author': 'Jakub Dorňák',
    'author_email': 'jakub.dornak@misli.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
